import React, { useEffect, useState } from 'react'
import Backdrop from '../Backdrop'
import Select from 'react-select'
import makeAnimated from 'react-select/animated'
import { Label } from '../ui/label'
import { Input } from '../ui/input'
import { Card, CardContent } from '../ui/card'
import { projectsTypes, technologies, technosTypes } from '@/types/types'
import { Textarea } from '../ui/textarea'
import { Button } from '../ui/button'
import { X } from 'lucide-react'
import { AxiosError } from 'axios'
import { toast } from 'react-toastify'
import axios from '@/config/axios'
import { CREATED, OK } from 'http-status'
import { patch, post } from '@/config/axiosFunctions'


const animatedComponents = makeAnimated()
type Props = {
  show_new_project_modal: () => void,
  remove_selected_project: () => void,
  selected_project: projectsTypes | null 
}

const ProjectForm = ({show_new_project_modal, remove_selected_project, selected_project}: Props) => {
  const [name, set_name] = useState<string>('')
  const [selected_technologies, set_selected_technologies] = useState<technosTypes[]>([])
  const [existing_technologies, set_existing_technologies] = useState<technosTypes[]>([])
  const [technologies, set_technologies] = useState<technosTypes[]>([])
  const [description, set_description] = useState<string>('')
  const [image, set_image] = useState<File>()
  const [url, set_url] = useState<string>('')
  const [is_submitting, set_is_submitting] = useState<boolean>(false)
  const [is_edit, set_is_edit] = useState(false)


  useEffect(() => {
    axios.get('/admin/technologies')
      .then(response => {
        let loaded_techno_array: technosTypes[] = []
        response.data['hydra:member'].map((technology: technologies) => {
          if (technology.status) {
            let techn_obj: technosTypes = {
              label: technology.name,
              value: technology.id
            }
            loaded_techno_array.push(techn_obj)
          }
          set_technologies(loaded_techno_array)
        })
      })
      .then(() => {
        if (selected_project !== null) {
          set_is_edit(true)
          set_name(selected_project.name)
          set_description(selected_project.description)
          set_url(selected_project.url)
          let selected_techno_array: technosTypes[] = []
          selected_project.technologies.map(technology => {
            let techn_obj: technosTypes = {
              label: technology.name,
              value: technology.id
            }
            selected_techno_array.push(techn_obj)
          })
          set_existing_technologies(selected_techno_array)
        }
      })
      .catch((err: AxiosError) => {
        toast.error('technologies not loaded')
      })
    }, [selected_project])
    
  const handle_submit = async (e: React.FormEvent) => {
    e.preventDefault()
    set_is_submitting(true)
    let selected_techno = selected_technologies
    if (
      selected_technologies.length === 0 &&
      is_edit
    ) {
      selected_techno = existing_technologies
    }
    
    const techno_id_arr: string[] = []

    selected_techno.map(techno => {
      techno_id_arr.push('/api/technologies/' + techno.value)
    })

    let request_body: {
      name: string, 
      url: string, 
      description: string, 
      technologies: string[], 
      image?: string
    } = {
      name,
      url: url.trim(),
      description: description.trim(),
      technologies: techno_id_arr,
    }
    
    if (is_edit) {
      if(image){
        const img_uri = await upload_image(image)
        request_body = {...request_body, image: img_uri}
      }
      try {
        const response = await patch('/admin/projects/' + selected_project?.id, request_body)
        if (response.status === OK) {
          set_is_submitting(false)
          toast.success('project updated')
          handle_close_modal()
        }
        
      } catch (err: any) {
        set_is_submitting(false)
        toast.error(err.message)
      }
    } else {
      if(image){
        const img_uri = await upload_image(image)
        request_body = {...request_body, image: img_uri} 
      }
      try {
        const response = await post('/admin/projects', request_body)
        if (response.status === CREATED) {
          set_is_submitting(false)
          toast.success('project created')
          handle_close_modal()
        }
      } catch (error : any) {
        set_is_submitting(false)
        toast.error(error.response.statusText)
      }
    }
  }
  const handle_close_modal = () => {
    set_is_edit(false)
    remove_selected_project()
    show_new_project_modal()
    set_name('')
    set_selected_technologies([])
    set_description('')
    set_image(undefined)
    set_url('')
    set_name('')
    set_name('')
  }
  const upload_image = async (file: File): Promise<string> => {
    let img_uri = ''
    const form_data = new FormData()
    form_data.append('file', file)
    const response = await axios.post('/mediaObjects', form_data, { headers: {'content-type': 'multipart/form-data'} })
    if (response.status === CREATED) {
      img_uri = response.data['@id']
    } else {
      set_is_submitting(true)
      toast.error('An error occured while uploading image')
    }
    return img_uri
  }

  return (
    <Backdrop>
      <Card className='bg-white py-4 lg:w-1/2'>
        <CardContent>
          <p className='flex justify-end'><span className='cursor-pointer' onClick={handle_close_modal}><X/></span></p>
          <div className="">
            <form onSubmit={handle_submit} className='space-y-3'>
              <div className="">
                <Label>Name</Label>
                <Input required onChange={(e) => set_name(e.target.value)} value={name} className='border-gray-500' />
              </div>
              <div className="">
                <Label>Technologies</Label>
                { existing_technologies.length > 0 && is_edit &&
                    <Select
                    required
                    closeMenuOnSelect={false}
                    components={animatedComponents}
                    onChange={(selected: any) => set_selected_technologies(selected) }
                    isMulti
                    options={technologies}
                    defaultValue={existing_technologies}
                  />
                }
                { !is_edit &&
                  <Select
                    required
                    closeMenuOnSelect={false}
                    components={animatedComponents}
                    onChange={(selected: any) => set_selected_technologies(selected) }
                    isMulti
                    options={technologies}
                  />
                }
              </div>
              <div className="">
                <Label>Description</Label>
                <Textarea required value={description} onChange={(e) => set_description(e.target.value) } placeholder="" className='border-gray-500'/>
              </div>
              <div className="">
                <Label>Url</Label>
                <Input required type='text' onChange={(e) => set_url(e.target.value)} value={url} className='border-gray-500' />
              </div>
              <div className="">
                <Label>Image</Label>
                <Input accept='image/png, image/jpg, image/jpeg' type='file' onChange={(e) => set_image(e.target.files?.[0])} className='border-gray-500' />
              </div>
              <div className="flex justify-between">
                <Button disabled={is_submitting} type='submit' className='bg-green-500 text-white' >Save</Button>
                <Button onClick={handle_close_modal} type='button' className='bg-red-500 text-white'>Cancel</Button>
              </div>
            </form>
          </div>
        </CardContent>
      </Card>
    </Backdrop>
  )
}

export default ProjectForm