import React, { useEffect, useState } from 'react'
import Backdrop from '../Backdrop'
import { Card, CardContent } from '../ui/card'
import { Label } from '@radix-ui/react-label'
import { Input } from '../ui/input'
import Select from 'react-select'
import { Button } from '../ui/button'
import makeAnimated from 'react-select/animated'
import { X } from 'lucide-react'
import { technologies, technosTypes } from '@/types/types'
import Image from 'next/image'
import { technology_type } from '@/utils/fakedb'
import axios from '@/config/axios'
import { toast } from 'react-toastify'
import { AxiosError } from 'axios'
import { CREATED, OK } from 'http-status'
import {get, post, patch} from '@/config/axiosFunctions'
import { SERVER_ADDRESS } from '@/utils/address'

const animatedComponents = makeAnimated()
const types = [
  {value: 'frontend', label:  'frontend'},
  {value: 'backend', label:  'backend'},
  {value: 'mobile', label:  'mobile'},
]
type Props = {
  handle_techno_modal: () => void,
  selected_tech: technologies | null,
  remove_selected_techno: () => void
}

const TechnologyForm = ({handle_techno_modal, selected_tech, remove_selected_techno}: Props) => {
  const [name, set_name] = useState<string>('')
  const [existing_type, set_existing_type] = useState<{value: string, label: string}>()
  const [selected_type, set_selected_type] = useState<{value: string, label: string}>()
  const [types, set_types] = useState<{value: string, label: string}[]>(technology_type)
  const [image, set_image] = useState<Blob>()
  const [image_url, set_image_url] = useState<string>('')
  const [is_submitting, set_is_submitting] = useState<boolean>(false)
  const [is_edit, set_is_edit] = useState(false)

  
  useEffect(() => {
    if (selected_tech) {
      set_is_edit(true)
      set_name(selected_tech.name)
      set_existing_type({value:selected_tech.type, label: selected_tech.type})
      set_image_url(selected_tech.logo?.filePath)
    }
  }, [selected_tech])
  
  const handle_submit = async (e: React.FormEvent) => {
    e.preventDefault()
    set_is_submitting(true)
    let selected_submited_type
    if(!selected_type){
      selected_submited_type = existing_type
    } else {
      selected_submited_type = selected_type
    }
    let request_body: {name: string, type: string, logo?: string} = {
      name: name,
      type: selected_submited_type?.value as string,
    }
    const form_data = new FormData()
    form_data.append('logo', image as Blob)

    if (is_edit) {
      if(image){
        const img_uri = await upload_image(image)
        request_body = {...request_body, logo: img_uri}
      }
      try {
        const response = await patch('/admin/technologies/' + selected_tech?.id, request_body)
        if (response.status === OK) {
          set_is_submitting(false)
          toast.success('project updated')
          handle_close_modal()
        }
        
      } catch (err: any) {
        set_is_submitting(false)
        toast.error(err.message)
      }
    } else {
      if(image){
        const img_uri = await upload_image(image)
        request_body = {...request_body, logo: img_uri} 
      }
      try {
        const response = await post('/admin/technologies', request_body)
        if (response.status === CREATED) {
          set_is_submitting(false)
          toast.success('project created')
          handle_close_modal()
        }
      } catch (err: any) {
        set_is_submitting(false)
        toast.error(err.message)
      }
    }
  }
  
  const handle_close_modal = () => {
    handle_techno_modal()
    remove_selected_techno()
    set_name('')
    set_image_url('')
    set_types([])
    set_image(undefined)
  }

  const upload_image = async (file: Blob): Promise<string> => {
    let img_uri = ''
    const form_data = new FormData()
    form_data.append('file', file)
    const response = await axios.post('/mediaObjects', form_data, { headers: {'content-type': 'multipart/form-data'} })
    if (response.status === CREATED) {
      img_uri = response.data['@id']
    } else {
      set_is_submitting(true)
      toast.error('An error occured while uploading image')
    }
    return img_uri
  }
  return (
    <Backdrop>
      <Card className='bg-white py-4 lg:w-1/2'>
        <CardContent>
          <p className='flex justify-end'><span className='cursor-pointer' onClick={handle_close_modal}><X/></span></p>
          <div className="">
            <form onSubmit={handle_submit} className='space-y-3'>
              <div className="">
                <Label>Name</Label>
                <Input onChange={(e) => set_name(e.target.value)} value={name} className='border-gray-500' />
              </div>
              <div className="">
                <Label>Type</Label>
                { existing_type !== undefined && is_edit &&
                  <Select
                    components={animatedComponents}
                    onChange={(selected: any) => set_selected_type(selected) }
                    options={types}
                    isSearchable
                    defaultValue={existing_type}
                  />
                }
                {
                 !is_edit && <Select
                  components={animatedComponents}
                  onChange={(selected: any) => set_selected_type(selected) }
                  options={types}
                  isSearchable
                />
                }
              </div>
              <div className="">
                <Label>Image</Label>
                <Input type='file' onChange={(e) => set_image(e.target.files?.[0]) } 
                  className='border-gray-500' />
              </div>
              {is_edit && <div>
                <Image alt='logo' src={SERVER_ADDRESS + image_url} width={100} height={100} />
              </div>}
              <div className="flex justify-between">
                <Button disabled={is_submitting} type='submit' className='bg-black text-white' >Save</Button>
                <Button onClick={handle_close_modal} type='button' className='bg-red-500 text-white'>Cancel</Button>
              </div>
            </form>
          </div>
        </CardContent>
      </Card>
    </Backdrop>
  )
}

export default TechnologyForm