import React, { useState } from 'react'
import {
  Table,
  TableBody,
  TableCell,
  TableHead,
  TableHeader,
  TableRow,
} from "@/components/ui/table"
import { projectsTypes } from '@/types/types'
import Image from 'next/image'
import { ArrowRightToLine, FileEdit, XCircle } from 'lucide-react'
import { convertNumberToAbbreviation } from '@/utils/number-format'
import default_image from '@/assets/default.jpeg'
import { Switch } from '@/components/ui/switch'
import ReactPaginate from 'react-paginate'
import Link from 'next/link'
import moment from 'moment'
import { SERVER_ADDRESS } from '@/utils/address'

type Props = {
  head: string[],
  projects: projectsTypes[],
  show_desciption_modal: (text: string) => void,
  update_status: (id: number, value: boolean) => void
  handle_selected_project: (project: projectsTypes) => void
}

const TableEntries = ({head, projects, show_desciption_modal, handle_selected_project, update_status}: Props) => {
  const itemsPerPage = 8
  const [itemOffset, setItemOffset] = useState(0)
  const endOffset = itemOffset + itemsPerPage
  const currentItems = projects.slice(itemOffset, endOffset);
  const pageCount = Math.ceil(projects.length / itemsPerPage);

  const handlePageClick = (event: any) => {
    const newOffset = (event.selected * itemsPerPage) % projects.length;
    setItemOffset(newOffset)
  }


  const table_head = head.map((h, index) => (
    <TableHead key={index} scope='col' className="text-center"><span className='capitalize font-semibold'>{h}</span></TableHead>
  ))
  const projects_row = currentItems.map((project, index) => (
    <TableRow key={index}>
      <TableCell className="text-center">{index + 1}</TableCell>
      <TableCell className="w-64">{project.name}</TableCell>
      <TableCell className="w-20">
        {project.image ? <Image alt={project.name} src={SERVER_ADDRESS + project.image.filePath as string} height={30} width={60} />:
        <Image alt={project.name} src={default_image.src} height={30} width={60} />}
      </TableCell>
      <TableCell className="w-72"><span onClick={() => show_desciption_modal(project.description)} className='text-blue-500 cursor-pointer'>{project.description.slice(0,30)} ...</span></TableCell>
      <TableCell className="text-center w-20">{project.technologies.length}</TableCell>
      <TableCell className="text-center w-20">{convertNumberToAbbreviation(project.visitorsCount)}</TableCell>
      <TableCell className="text-center w-20">{<Switch defaultChecked={project.status} onClick={() => update_status(project.id, project.status)} />}</TableCell>
      <TableCell className="text-center">{moment(project.createdAt).format('LL')}</TableCell>
      <TableCell className="text-center">{moment(project.updatedAt).format('LL')}</TableCell>
      <TableCell className="">
        <div className="flex space-x-3 justify-center">
          <span className='cursor-pointer'  onClick={() => handle_selected_project(project)}>
            <FileEdit className='text-yellow-400 hover:scale-125 ease-in' />
          </span>
          <span className='cursor-pointer'>
            <XCircle className='text-red-500 hover:scale-125 ease-in' />
          </span>
          <Link passHref target='_blank' href={project.url}>
            <ArrowRightToLine className='text-green-500 hover:scale-125 ease-in' />
          </Link>
        </div>
      </TableCell>
    </TableRow>
  ))
  return (
    <>
      <Table>
        <TableHeader>
          <TableRow>
            {table_head}
          </TableRow>
        </TableHeader>
        <TableBody>
          {projects.length > 0 ? projects_row :
           <TableRow>
            <TableCell colSpan={1} className="text-center">No records found</TableCell>
          </TableRow>  }
        </TableBody>
      </Table>
      <ReactPaginate
        breakLabel="..."
        nextLabel="next >"
        onPageChange={handlePageClick}
        pageRangeDisplayed={5}
        pageCount={pageCount}
        previousLabel="< previous"
        renderOnZeroPageCount={null}
        containerClassName="flex items-center space-x-3 my-5"
        pageLinkClassName="p-1 px-2 rounded-md text-[crimson]"
        activeClassName="flex bg-[crimson] justify-center items-center bg-white text-[crimson] border-[crimson] border"
      />
    </>
  )
}

export default TableEntries